@@ -1,11 +1,13 @@
 package GD::Graph::Polar;
 use strict;
 use warnings;
+use base qw{Package::New};
 use Geo::Constants qw{PI};
 use Geo::Functions qw{rad_deg};
 use GD qw{gdSmallFont};
+use List::Util qw{first};
 
-our $VERSION="0.16";
+our $VERSION="0.17";
 
 =head1 NAME
 
@@ -46,86 +48,16 @@ This package is a wrapper arround GD to produce polar graphs with an easy interf
 
 The new constructor. 
 
-  my $obj = GD::Graph::Polar->new(               #default values
-                                  size=>480,     #width and height in pixels
-                                  radius=>1,     #scale of the radius
-                                  ticks=>10,     #number of major ticks
-                                  border=>2,     #pixel border around graph
-                                  rgbfile=>"/usr/X11R6/lib/X11/rgb.txt"
+  my $obj = GD::Graph::Polar->new(           #default values
+                                  size    => 480,    #width and height in pixels
+                                  radius  => 1,      #scale of the radius
+                                  ticks   => 10,     #number of major ticks
+                                  border  => 2,      #pixel border around graph
+                                  rgbfile => "/usr/X11R6/lib/X11/rgb.txt"
                                  );
 
-=cut
-
-sub new {
-  my $this = shift();
-  my $class = ref($this) || $this;
-  my $self = {};
-  bless $self, $class;
-  $self->initialize(@_);
-  return $self;
-}
-
 =head1 METHODS
 
-=head2 initialize
-
-=cut
-
-sub initialize {
-  my $self = shift();
-  my $param = {@_};
-  $self->{'size'}=$param->{'size'} || 480;
-  $self->{'radius'}=$param->{'radius'} || 1;
-  $self->{'ticks'}=defined($param->{'ticks'}) ? $param->{'ticks'} : 10;
-  $self->{'border'}=defined($param->{'border'}) ? $param->{'border'} : 2;
-  $self->{'object'}=GD::Image->new($self->{'size'}, $self->{'size'});
-  eval 'use Graphics::ColorNames';
-  unless($@) {
-    my $rgb;
-    foreach (qw{
-                 /usr/share/X11/rgb.txt
-                 /usr/X11R6/lib/X11/rgb.txt
-                 /etc/X11/rgb.txt
-                 ./rgb.txt
-                 ../rgb.txt
-               }) {
-      $rgb=$_ if -r;
-    }
-    $self->{'rgbfile'}=$param->{'rgbfile'} || $rgb;
-    die('Error: Cannot read rgb.txt file "'. $self->{'rgbfile'}. '"') unless -r $self->{'rgbfile'};
-    $self->{'ColorNames'}=Graphics::ColorNames->new($self->{'rgbfile'});
-  }
-
-  # make the background transparent and interlaced
-  $self->{'object'}->transparent($self->color([255,255,255]));
-  $self->{'object'}->interlaced('true');
-  
-  # Put a frame around the picture
-  $self->{'object'}->rectangle(0, 0, $self->{'size'}-1, $self->{'size'}-1, $self->color([0,0,0]));
-
-  $self->color([192,192,192]);
-  if ($self->{'ticks'} > 0) {
-    foreach (0..$self->{'ticks'}) {
-      my $c=$self->{'size'} / 2;
-      my $r=$self->_width * $_ / $self->{'ticks'};
-      $self->{'object'}->arc($c,$c,$r,$r,0,360,$self->color);
-    }
-  }
-
-  $self->{'object'}->line($self->{'size'}/2,
-                          $self->{'border'},
-                          $self->{'size'}/2,
-                          $self->{'size'}-$self->{'border'},
-                          $self->color);
-  $self->{'object'}->line($self->{'border'},
-                          $self->{'size'}/2,
-                          $self->{'size'}-$self->{'border'},
-                          $self->{'size'}/2,
-                          $self->color);
-  $self->font(gdSmallFont);
-  $self->color([0,0,0]);
-}
-
 =head2 addPoint
 
 Method to add a point to the graph.
@@ -135,9 +67,9 @@ Method to add a point to the graph.
 =cut
 
 sub addPoint {
-  my $self = shift();
-  my $r=shift();
-  my $t=rad_deg(shift());
+  my $self = shift;
+  my $r    = shift;
+  my $t    = rad_deg(shift());
   return $self->addPoint_rad($r,$t);
 }
 
@@ -150,12 +82,12 @@ Method to add a point to the graph.
 =cut
 
 sub addPoint_rad {
-  my $self = shift();
-  my $r=shift();
-  my $t=shift();
+  my $self = shift;
+  my $r    = shift;
+  my $t    = shift;
   my ($x, $y)=$self->_imgxy_rt_rad($r,$t);
   my $icon=7;
-  return $self->{'object'}->arc($x,$y,$icon,$icon,0,360,$self->color);
+  return $self->gdimage->arc($x,$y,$icon,$icon,0,360,$self->color);
 }
 
 =head2 addGeoPoint
@@ -167,9 +99,9 @@ Method to add a point to the graph.
 =cut
 
 sub addGeoPoint {
-  my $self = shift();
-  my $r=shift();
-  my $t=rad_deg(shift());
+  my $self = shift;
+  my $r    = shift;
+  my $t    = rad_deg(shift());
   return $self->addGeoPoint_rad($r,$t);
 }
 
@@ -182,9 +114,9 @@ Method to add a point to the graph.
 =cut
 
 sub addGeoPoint_rad {
-  my $self = shift();
-  my $r=shift();
-  my $t=PI()/2-shift();
+  my $self = shift;
+  my $r    = shift;
+  my $t    = PI()/2-shift();
   return $self->addPoint_rad($r,$t);
 }
 
@@ -197,11 +129,11 @@ Method to add a line to the graph.
 =cut
 
 sub addLine {
-  my $self = shift();
-  my $r0=shift();
-  my $t0=rad_deg(shift());
-  my $r1=shift();
-  my $t1=rad_deg(shift());
+  my $self = shift;
+  my $r0   = shift;
+  my $t0   = rad_deg(shift());
+  my $r1   = shift;
+  my $t1   = rad_deg(shift());
   return $self->addLine_rad($r0=>$t0, $r1=>$t1);
 }
 
@@ -214,14 +146,14 @@ Method to add a line to the graph.
 =cut
 
 sub addLine_rad {
-  my $self = shift();
-  my $r0=shift();
-  my $t0=shift();
-  my $r1=shift();
-  my $t1=shift();
+  my $self = shift;
+  my $r0   = shift;
+  my $t0   = shift;
+  my $r1   = shift;
+  my $t1   = shift;
   my ($x0=>$y0)=$self->_imgxy_rt_rad($r0=>$t0);
   my ($x1=>$y1)=$self->_imgxy_rt_rad($r1=>$t1);
-  return $self->{'object'}->line($x0, $y0, $x1, $y1, $self->color);
+  return $self->gdimage->line($x0, $y0, $x1, $y1, $self->color);
 }
 
 =head2 addGeoLine
@@ -233,11 +165,11 @@ Method to add a line to the graph.
 =cut
 
 sub addGeoLine {
-  my $self = shift();
-  my $r0=shift();
-  my $t0=rad_deg(shift());
-  my $r1=shift();
-  my $t1=rad_deg(shift());
+  my $self = shift;
+  my $r0   = shift;
+  my $t0   = rad_deg(shift());
+  my $r1   = shift;
+  my $t1   = rad_deg(shift());
   return $self->addGeoLine_rad($r0=>$t0, $r1=>$t1);
 }
 
@@ -250,11 +182,11 @@ Method to add a line to the graph.
 =cut
 
 sub addGeoLine_rad {
-  my $self = shift();
-  my $r0=shift();
-  my $t0=PI()/2-shift();
-  my $r1=shift();
-  my $t1=PI()/2-shift();
+  my $self = shift;
+  my $r0   = shift;
+  my $t0   = PI()/2-shift();
+  my $r1   = shift;
+  my $t1   = PI()/2-shift();
   return $self->addLine_rad($r0=>$t0, $r1=>$t1);
 }
 
@@ -267,11 +199,11 @@ Method to add an arc to the graph.
 =cut
 
 sub addArc {
-  my $self = shift();
-  my $r0=shift();
-  my $t0=rad_deg(shift());
-  my $r1=shift();
-  my $t1=rad_deg(shift());
+  my $self = shift;
+  my $r0   = shift;
+  my $t0   = rad_deg(shift());
+  my $r1   = shift;
+  my $t1   = rad_deg(shift());
   return $self->addArc_rad($r0=>$t0, $r1=>$t1);
 }
 
@@ -284,15 +216,15 @@ Method to add an arc to the graph.
 =cut
 
 sub addArc_rad {
-  my $self = shift();
-  my $r0=shift();
-  my $t0=shift();
-  my $r1=shift();
-  my $t1=shift();
-  my $m=($r1-$r0) / ($t1-$t0);
-  my $inc=0.02; #is this good?
-  my $steps=int(($t1-$t0) / $inc);
-  my @array=();
+  my $self  = shift;
+  my $r0    = shift;
+  my $t0    = shift;
+  my $r1    = shift;
+  my $t1    = shift;
+  my $m     = ($r1-$r0) / ($t1-$t0);
+  my $inc   = 0.02; #is this good?
+  my $steps = int(($t1-$t0) / $inc);
+  my @array = ();
   foreach (0..$steps) {
     my $t=$_ / $steps * ($t1-$t0) + $t0;
     my $r=$r0 + $m * ($t-$t0);
@@ -314,11 +246,11 @@ Method to add an arc to the graph.
 =cut
 
 sub addGeoArc {
-  my $self = shift();
-  my $r0=shift();
-  my $t0=rad_deg(shift());
-  my $r1=shift();
-  my $t1=rad_deg(shift());
+  my $self = shift;
+  my $r0   = shift;
+  my $t0   = rad_deg(shift());
+  my $r1   = shift;
+  my $t1   = rad_deg(shift());
   return $self->addGeoArc_rad($r0=>$t0, $r1=>$t1);
 }
 
@@ -331,11 +263,11 @@ Method to add an arc to the graph.
 =cut
 
 sub addGeoArc_rad {
-  my $self = shift();
-  my $r0=shift();
-  my $t0=PI()/2-shift();
-  my $r1=shift();
-  my $t1=PI()/2-shift();
+  my $self = shift;
+  my $r0   = shift;
+  my $t0   = PI()/2-shift();
+  my $r1   = shift;
+  my $t1   = PI()/2-shift();
   return $self->addArc_rad($r0=>$t0, $r1=>$t1);
 }
 
@@ -346,10 +278,10 @@ Method to add a string to the graph.
 =cut
 
 sub addString {
-  my $self=shift();
-  my $r=shift();
-  my $t=rad_deg(shift());
-  my $string=shift();
+  my $self   = shift;
+  my $r      = shift;
+  my $t      = rad_deg(shift());
+  my $string = shift;
   return $self->addString_rad($r=>$t, $string);
 }
 
@@ -360,12 +292,12 @@ Method to add a string to the graph.
 =cut
 
 sub addString_rad {
-  my $self=shift();
-  my $r=shift();
-  my $t=shift();
-  my $string=shift();
+  my $self   = shift;
+  my $r      = shift;
+  my $t      = shift;
+  my $string = shift;
   my ($x=>$y)=$self->_imgxy_rt_rad($r=>$t);
-  return $self->{'object'}->string($self->font, $x, $y, $string, $self->color);
+  return $self->gdimage->string($self->font, $x, $y, $string, $self->color);
 }
 
 =head2 addGeoString
@@ -375,10 +307,10 @@ Method to add a string to the graph.
 =cut
 
 sub addGeoString {
-  my $self=shift();
-  my $r=shift();
-  my $t=rad_deg(shift());
-  my $string=shift();
+  my $self   = shift;
+  my $r      = shift;
+  my $t      = rad_deg(shift());
+  my $string = shift;
   return $self->addGeoString_rad($r=>$t, $string);
 }
 
@@ -389,13 +321,83 @@ Method to add a string to the graph.
 =cut
 
 sub addGeoString_rad {
-  my $self=shift();
-  my $r=shift();
-  my $t=PI()/2-shift();
-  my $string=shift();
+  my $self   = shift;
+  my $r      = shift;
+  my $t      = PI()/2-shift();
+  my $string = shift;
   return $self->addString_rad($r=>$t, $string);
 }
 
+=head1 Objects
+
+=head2 gdimage
+
+Returns a L<GD> object
+
+=cut
+
+sub gdimage {
+  my $self=shift;
+  $self->{'gdimage'}=shift if @_; #set a base chart or watermark
+  unless ($self->{'gdimage'}) {
+    $self->{'gdimage'}=GD::Image->new($self->size, $self->size);
+
+    # make the background transparent and interlaced
+    $self->gdimage->transparent($self->color([255,255,255]));
+    $self->gdimage->interlaced('true');
+    
+    # Put a frame around the picture
+    $self->gdimage->rectangle(0, 0, $self->size - 1, $self->size - 1, $self->color([0,0,0]));
+  
+    if ($self->ticks > 0) {
+      $self->color([192,192,192]);
+      foreach (0 .. $self->ticks) {
+        my $c=$self->size / 2;
+        my $r=$self->_width * $_ / $self->ticks;
+        $self->gdimage->arc($c,$c,$r,$r,0,360,$self->color);
+      }
+    }
+  
+    $self->color([192,192,192]);
+    $self->gdimage->line($self->size/2,
+                            $self->border,
+                            $self->size/2,
+                            $self->size-$self->border,
+                            $self->color);
+    $self->gdimage->line($self->border,
+                            $self->size/2,
+                            $self->size-$self->border,
+                            $self->size/2,
+                            $self->color);
+
+    #default to black pen color
+    $self->color([0,0,0]);
+  }
+  return $self->{'gdimage'};
+}
+
+=head2 gcnames
+
+Returns a L<Graphics::ColorNames> object
+
+=cut
+
+sub gcnames {
+  my $self=shift;
+  unless (defined $self->{'gcnames'}) {
+    eval 'use Graphics::ColorNames';
+    if ($@) {
+      die("Error: Cannot load Graphics::ColorNames");
+    } else {
+      my $file=$self->rgbfile; #stringify for object support
+      $self->{'gcnames'}=Graphics::ColorNames->new("$file");
+    }
+  }
+  return $self->{'gcnames'};
+}
+
+=head1 Properties
+
 =head2 color
 
 Method to set or return the current drawing color
@@ -407,19 +409,19 @@ Method to set or return the current drawing color
 =cut
 
 sub color {
-  my $self = shift();
+  my $self=shift;
   if (@_) {
-    my $color=shift();
-    if (ref($color) eq "ARRAY") {
-      $self->{'color'}=$self->{'object'}->colorAllocate(@{$color});
+    my $color=shift;
+    if (ref($color) eq 'ARRAY') {
+      my ($r, $g, $b)= @$color;
+      $self->{'color'}=$self->{'colors'}->{$r}->{$g}->{$b}||=$self->gdimage->colorAllocate(@$color);
     } else {
-      if (defined($self->{'ColorNames'})) {
-        $self->{'color'}=
-          $self->{'object'}->colorAllocate($self->{'ColorNames'}->rgb($color))
-            || $self->{'object'}->colorAllocate(0,0,0);
+      if ($self->gcnames) {
+        my @rgb=$self->gcnames->rgb($color);
+        @rgb=(0,0,0) unless scalar(@rgb) == 3;
+        $self->{'color'}=$self->color(\@rgb);
       } else {
-        warn(qq{Warning: Graphics::ColorNames unavailable. Unknown color($color); Using [0,0,0] (black).\n});
-        $self->{'color'}=$self->{'object'}->colorAllocate(0,0,0);
+        $self->{'color'}=$self->color([0,0,0]);
       }
     }
   }
@@ -437,11 +439,80 @@ Method to set or return the current drawing font (only needed by the very few)
 =cut
 
 sub font {
-  my $self = shift();
-  if (@_) { $self->{'font'} = shift() } #sets value
+  my $self=shift;
+  $self->{'font'}=shift if @_;
+  $self->{'font'}=gdSmallFont unless $self->{'font'};
   return $self->{'font'};
 }
 
+=head2 size
+
+Sets or returns the width and height of the graph in pixels.
+
+=cut
+
+sub size {
+  my $self=shift;
+  $self->{'size'}=shift if @_;
+  $self->{'size'}=480 unless $self->{'size'};
+  return $self->{'size'};
+}
+
+=head2 radius
+
+Sets or returns the radius of the Graph
+
+=cut
+
+sub radius {
+  my $self=shift;
+  $self->{'radius'}=shift if @_;
+  $self->{'radius'}=1 unless $self->{'radius'};
+  return $self->{'radius'};
+}
+
+=head2 border
+
+=cut
+
+sub border {
+  my $self=shift;
+  $self->{'border'}=shift if @_;
+  $self->{'border'}=2 unless defined($self->{'border'});
+  return $self->{'border'};
+}
+
+=head2 ticks
+
+=cut
+
+sub ticks {
+  my $self=shift;
+  $self->{'ticks'}=shift if @_;
+  $self->{'ticks'}=10
+    unless defined($self->{'ticks'});
+  return $self->{'ticks'};
+}
+
+=head2 rgbfile
+
+Sets or returns an RGB file.
+
+Note: This method will search in a few locations for a file.
+
+=cut
+
+sub rgbfile {
+  my $self=shift;
+  $self->{'rgbfile'}=shift if @_;
+  unless (defined $self->{'rgbfile'}) {
+    $self->{'rgbfile'}="rgb.txt";
+    my $rgb=first {-r} (qw{/etc/X11/rgb.txt /usr/share/X11/rgb.txt /usr/X11R6/lib/X11/rgb.txt ../rgb.txt});
+    $self->{'rgbfile'}=$rgb if $rgb;
+  }
+  return $self->{'rgbfile'};
+}
+
 =head2 draw
 
 Method returns a PNG binary blob.
@@ -451,8 +522,8 @@ Method returns a PNG binary blob.
 =cut
 
 sub draw {
-  my $self=shift();
-  return $self->{'object'}->png;
+  my $self=shift;
+  return $self->gdimage->png;
 }
 
 #=head2 _scale
@@ -462,9 +533,9 @@ sub draw {
 #=cut
 
 sub _scale {
-  my $self=shift();
-  my $r=shift();
-  return $self->_width / 2 / $self->{'radius'} * $r;
+  my $self = shift;
+  my $r    = shift;
+  return $self->_width / 2 / $self->radius * $r;
 }
 
 #=head2 _width
@@ -474,8 +545,8 @@ sub _scale {
 #=cut
 
 sub _width {
-  my $self=shift();
-  return $self->{'size'} - $self->{'border'} * 2;
+  my $self=shift;
+  return $self->size - $self->border * 2;
 }
 
 #=head2 _imgxy_xy
@@ -487,12 +558,12 @@ sub _width {
 #=cut
 
 sub _imgxy_xy {
-  my $self = shift();
-  my $x=shift();
-  my $y=shift();
-  my $sz=$self->_width;
-  $x=$sz/2 + $x + $self->{'border'};
-  $y=$sz/2 - $y + $self->{'border'};
+  my $self = shift;
+  my $x    = shift();
+  my $y    = shift();
+  my $sz   = $self->_width;
+  $x       = $sz/2 + $x + $self->border;
+  $y       = $sz/2 - $y + $self->border;
   return ($x, $y);
 }
 
@@ -503,11 +574,11 @@ sub _imgxy_xy {
 #=cut
 
 sub _xy_rt_rad {
-  my $self=shift();
-  my $r=shift();
-  my $t=shift();
-  my $x=$r*cos($t);
-  my $y=$r*sin($t);
+  my $self = shift;
+  my $r    = shift;
+  my $t    = shift;
+  my $x    = $r*cos($t);
+  my $y    = $r*sin($t);
   return ($x, $y);
 }
 
@@ -518,9 +589,9 @@ sub _xy_rt_rad {
 #=cut
 
 sub _imgxy_rt_rad {
-  my $self = shift();
-  my $r=shift();
-  my $t=shift();
+  my $self = shift;
+  my $r    = shift;
+  my $t    = shift;
   my ($x,$y)=$self->_xy_rt_rad($self->_scale($r), $t);
   return $self->_imgxy_xy($x, $y);
 }
