@@ -10,7 +10,7 @@ example-text.pl - GD::Graph::Polar example
 
 =head1 SAMPLE OUTPUT
 
-L<http://search.cpan.org/src/MRDVT/GD-Graph-Polar-0.16/bin/example-text.png>
+L<http://search.cpan.org/src/MRDVT/GD-Graph-Polar-0.16/scripts/example-text.png>
 
 =cut
 
